#include "testlib.h"
#include <vector>
#include <set>

using namespace std;

typedef unsigned int uint;
typedef pair<int, int> pii;

const int limit = 1000'000'000;

const vector<pii> shifts{
        {-1, 0},
        {0,  -1},
        {1,  -1},
        {1,  0},
        {0,  1},
        {-1, 1}
};

int check(InStream &in, vector<pii> const &input) {
    int n = input.size();
    set<pii> cells(input.begin(), input.end());

    int k = in.readInt(0, n, "k");
    for (int i = 0; i < k; i++) {
        int x = in.readInt(-limit, limit, format("x_%d", i + 1)),
                y = in.readInt(-limit, limit, format("y_%d", i + 1));
        in.ensuref(cells.count({x, y}) > 0,
                   "Cell (%d, %d) was not in the starting set or is already removed", x, y);
        cells.erase({x, y});
    }

    for (auto const &p: cells) {
        uint nei = 0u;
        for (int i = 0; i < shifts.size(); i++) {
            if (cells.count({p.first + shifts[i].first, p.second + shifts[i].second})) {
                in.ensuref(i == 0 || ((nei >> (i - 1u)) & 1u) == 0,
                           "Triangle spot present around (%d, %d)", p.first, p.second);
                nei |= (1u << i);
            }
        }
        in.ensuref((nei & 0b100001) < 0b100001,
                   "Triangle spot present around (%d, %d)", p.first, p.second);
    }

    return k;
}

int main(int argc, char *argv[]) {

    registerTestlibCmd(argc, argv);

    int n = inf.readInt();
    double cost = inf.readInt();
    double gamma = inf.readDouble();

    vector<pii> xy(n);
    for (int i = 0; i < n; i++) {
        xy[i].first = inf.readInt();
        xy[i].second = inf.readInt();
    }

    double ja = check(ans, xy), pa = check(ouf, xy);
    double ok = ja / pa;
    double score = ok < gamma ? 0.0 : cost * min(1.0, ok);
    // quitp(static_cast<int>(score * 100 / max(cost, 1.0)), "valid answer");
    quitp(static_cast<int>(score), "valid answer");

}