#include "testlib.h"

using namespace std;

typedef pair<int, int> pii;

inline int color(pii const &c) {
    int col = (c.second - c.first) % 3;
    return col < 0 ? col + 3 : col;
}

int const lim = 1e9;

inline pii around(pii const &p, int it) {
    // @formatter:off
    static const vector<pii> shifts{
            {-1, 0}, {0,  -1}, {1,  -1},
            {1,  0}, {0,  1}, {-1, 1}
    };
    // @formatter:on
    return {p.first + shifts[it].first, p.second + shifts[it].second};
}

inline int rsh(int t) {
    return (t ^ (t << 1u) ^ (t >> 4u)) * 7 % 239;
}

int main(int argc, char *argv[]) {

    registerGen(argc, argv, 1);
    int n = opt<int>("n"), cost = opt<int>("cost");
    double gamma = opt<double>("gamma");
    int spots = opt<int>("spots"), maxd = opt<int>("maxd");

    println(n, cost, gamma);
    auto split = rnd.distinct(spots - 1, 1, n - 1);
    split.push_back(n);
    sort(split.begin(), split.end());

    vector<pii> cells;
    set<pii> cells_set;
    auto deg = [&](pii const &c) {
        int cnt = 0;
        for (int i = 0; i < 6; i++)
            if (cells_set.count(around(c, i)))
                cnt++;
        return cnt;
    };

    for (int i = 0; i < spots; i++) {
        auto cmp = [&](pii const &c1, pii const &c2) {
            if (color(c1) != color(c2))
                return (color(c1) + i) % 3 < (color(c2) + i) % 3;
            if (rsh(c1.first) != rsh(c2.first))
                return rsh(c1.first) < rsh(c2.first);
            if (rsh(c1.second) != rsh(c2.second))
                return rsh(c1.second) < rsh(c2.second);
            return c1 < c2;
        };
        set<pii, decltype(cmp)> q(cmp);
        int x = rnd.next(-lim, lim), y = rnd.next(-lim, lim);
        pii s(x, y);
        q.insert(s);
        cells.push_back(s);
        cells_set.insert(s);

        while (cells.size() < split[i]) {
            if (q.empty())
                q.insert(cells[rnd.next(cells.size())]);
            pii c = *q.begin();
            q.erase(*q.begin());
            int t = maxd - deg(c);

            while (cells.size() < split[i] && t--) {
                pii c2 = around(c, rnd.next(6));
                while (cells_set.count(c2))
                    c2 = around(c, rnd.next(6));
                if (!cells_set.count(c2) && deg(c2) <= maxd) {
                    bool ok = true;
                    for (int k = 0; k < 6; k++)
                        if (deg(around(c2, k)) >= maxd)
                            ok = false;
                    if (ok) {
                        cells.push_back(c2);
                        cells_set.insert(c2);
                        q.insert(c2);
                    }
                }
            }
        }
    }

    shuffle(cells.begin(), cells.end());
    for (auto const &c: cells) println(c.first, c.second);

}