#include "testlib.h"
#include <unordered_set>
#include <functional>

using namespace std;

typedef pair<int, int> pii;

template<typename T1, typename T2>
struct phash {
    size_t operator()(pair<T1, T2> const &p) const {
        return (hash<T1>()(p.first) * 177013u) ^ hash<T2>()(p.second);
    }
};

inline pii around(pii const &p, int it) {
    // @formatter:off
    static const vector<pii> shifts{
            {-1, 0}, {0,  -1}, {1,  -1},
            {1,  0}, {0,  1}, {-1, 1}
    };
    // @formatter:on
    return {p.first + shifts[it].first, p.second + shifts[it].second};
}

int main(int argc, char *argv[]) {

    registerValidation(argc, argv);

    int n = inf.readInt(1, 100000, "n");
    inf.readSpace();
    int c = inf.readInt(0, 5, "cost");
    inf.readSpace();
    double gamma = inf.readDouble(0.0, 1.0, "gamma");
    inf.readEoln();

    // if (c == 2)
    //    ensuref(n <= 3, "c = 2 can't have n = %d > 3", n);
    //else if (c == 3)
    //    ensuref(n <= 18, "c = 2 can't have n = %d > 18", n);
    //else if (c == 4 && gamma < 0.5)
    //    ensuref(n <= 1000, "c = 2 can't have n = %d > 1000", n);

    unordered_set<pii, phash<int, int>> cells;
    const int limit = 1000000000;
    for (int i = 0; i < n; i++) {
        int x = inf.readInt(-limit, limit, format("x_%d", i + 1));
        inf.readSpace();
        int y = inf.readInt(-limit, limit, format("y_%d", i + 1));
        inf.readEoln();

        pii p(x, y);
        ensuref(cells.count(p) == 0, "repeating cell %d, %d", x, y);
        cells.insert(p);
    }

    if (c == 4 && gamma < 0.5) {
        unordered_set<pii, phash<int, int>> vis;
        function<void(pii const &)> dfs = [&](pii const &v) {
            vis.insert(v);
            for (int i = 0; i < 6; i++) {
                auto u = around(v, i);
                if (vis.count(u) == 0 && cells.count(u) > 0)
                    dfs(u);
            }
        };
        dfs(*cells.begin());
        ensuref(vis.size() == n, "set should be connected");
    } else if (c == 4) {
        for (auto const &p : cells) {
            int cnt = 0;
            for (int i = 0; i < 6; i++)
                if (cells.count(around(p, i)))
                    cnt++;
            ensuref(cnt <= 4, "cell %d, %d has %d > 4 neighbours", p.first, p.second, cnt);
        }
    }

    inf.readEof();

}